import openpyxl as xl
import re


class CellAddressModel():
    """セル・アドレス・モデル。
    """


    _code_pattern = re.compile(r'(\w+)(\d+)')


    @classmethod
    def from_code(clazz, code):
        result = clazz._code_pattern.match(code)

        if not result:
            raise ValueError(f"コード読取失敗。 {code=}")

        column_th   = xl.utils.column_index_from_string(result.group(1))
        row_th      = int(result.group(2))

        return CellAddressModel(
                column_th   = column_th,
                row_th      = row_th)


    def __init__(self, column_th, row_th):
        self._column_th = column_th
        self._row_th = row_th


    @property
    def column_th(self):
        return self._column_th


    @property
    def row_th(self):
        return self._row_th


    def to_code(self):
        return f"{xl.utils.get_column_letter(self._column_th)}{self._row_th}"


    def dump_string(self):
        return f"""{{"column_th"={self._column_th}, "row_th"={self._row_th}}}"""



import openpyxl as xl


class GraphPaperRenderer():
    """方眼紙を描画します。
    """

    @staticmethod
    def render(left_th, top_th, width, height, ws):
        """描画。

        Parameters
        ----------
        left_th : int
            開始列。1から始まる数。
        top_th : int
            開始行。1から始まる数。
        width : int
            横のセル数。
        height : int
            縦のセル数。
        ws : Worksheet
            ワークシート。
        """

        # 行の横幅
        for column_th in range(left_th, width+1):
            column_letter = xl.utils.get_column_letter(column_th)
            ws.column_dimensions[column_letter].width = 2.7    # 2.7 characters = about 30 pixels

        # 列の高さ
        for row_th in range(top_th, height+1):
            ws.row_dimensions[row_th].height = 15    # 15 points = about 30 pixels



import openpyxl as xl


class ColumnLetterIterator:


    def __init__(self, start, end):
        """初期化。

        Parameters
        ----------
        start : str
            開始列のアルファベット。
        end : str
            終了列のアルファベット。この終了列自身は含まない。
        """
        self._current_th = xl.utils.column_index_from_string(start)
        self._end_th = xl.utils.column_index_from_string(end)


    def __iter__(self):
        return self


    def __next__(self):
        if self._current_th < self._end_th:
            result = self._current_th
            self._current_th += 1
            return xl.utils.get_column_letter(result)
        else:
            raise StopIteration()
